#include "CommandBuffer.h"

/*********************************************************************
 * Returns the first free slot of the buffer.
 * If the buffer is full, -1 will be returned.
*********************************************************************/
int CommandBuffer::findFirstFreeSlot()
{
  for (int index = 0; index < BUFFER_SIZE; index++)
  {
    if (_commandBuffer[index] == NULL)
    {
      return index;
    }
  }
  return -1; // none found
}

/*********************************************************************
 * Adds new command to buffer.
*********************************************************************/
void CommandBuffer::add(Command *cmd)
{
  int freeSlot = findFirstFreeSlot();
  if (freeSlot < 0)
  { // If buffer is full
    Serial.println(F("Err: CmdBuffer full!"));
    return;
  }

  _commandBuffer[freeSlot] = cmd;
}

/*********************************************************************
 * Empties the buffer by calling remove for every slot.
*********************************************************************/
void CommandBuffer::clear()
{
  for (int index = 0; index < BUFFER_SIZE; index++)
  {
    remove(index);
  }
}

/*********************************************************************
 * Destructs object and removes reference to Command from buffer.
*********************************************************************/
void CommandBuffer::remove(int index)
{
  delete _commandBuffer[index];
  _commandBuffer[index] = NULL;
}

/*********************************************************************
 * Returns value in buffer on given index.
*********************************************************************/
Command *CommandBuffer::get(int index)
{
  return _commandBuffer[index];
}

int CommandBuffer::size()
{
  return BUFFER_SIZE;
}

/*********************************************************************
 * Returns true if all elements in buffer are NULL.
*********************************************************************/
bool CommandBuffer::isEmpty()
{
  for (int index = 0; index < BUFFER_SIZE; index++)
  {
    if (_commandBuffer[index] != NULL)
    {
      return false;
    }
  }
  return true;
}
